#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include "Yajin.h"

#include "images/up1_bsd.xpm"
#include "images/up2_bsd.xpm"
#include "images/upright1_bsd.xpm"
#include "images/upright2_bsd.xpm"
#include "images/right1_bsd.xpm"
#include "images/right2_bsd.xpm"
#include "images/dwright1_bsd.xpm"
#include "images/dwright2_bsd.xpm"
#include "images/down1_bsd.xpm"
#include "images/down2_bsd.xpm"
#include "images/dwleft1_bsd.xpm"
#include "images/dwleft2_bsd.xpm"
#include "images/left1_bsd.xpm"
#include "images/left2_bsd.xpm"
#include "images/upleft1_bsd.xpm"
#include "images/upleft2_bsd.xpm"

#define XPM_U0  up1_bsd_xpm
#define XPM_U1  up2_bsd_xpm
#define XPM_UR0 upright1_bsd_xpm
#define XPM_UR1 upright2_bsd_xpm
#define XPM_R0  right1_bsd_xpm
#define XPM_R1  right2_bsd_xpm
#define XPM_DR0 dwright1_bsd_xpm
#define XPM_DR1 dwright2_bsd_xpm
#define XPM_D0  down1_bsd_xpm
#define XPM_D1  down2_bsd_xpm
#define XPM_DL0 dwleft1_bsd_xpm
#define XPM_DL1 dwleft2_bsd_xpm
#define XPM_L0  left1_bsd_xpm
#define XPM_L1  left2_bsd_xpm
#define XPM_UL0 upleft1_bsd_xpm
#define XPM_UL1 upleft2_bsd_xpm

Yajin::Yajin(void)
{
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    CreatePixmaps();
    gtk_widget_shape_combine_mask(window, pixmap_u[0]->mask, 0, 0);
    gtk_widget_show(window);
    flag_step = 0;
    x = y = 0;
}

Yajin::~Yajin(void)
{
    free(pixmap_u[0]);  free(pixmap_u[1]);
    free(pixmap_ur[0]); free(pixmap_ur[1]);
    free(pixmap_r[0]);  free(pixmap_r[1]);
    free(pixmap_dr[0]); free(pixmap_dr[1]);
    free(pixmap_d[0]);  free(pixmap_d[1]);
    free(pixmap_dl[0]); free(pixmap_dl[1]);
    free(pixmap_l[0]);  free(pixmap_l[1]);
    free(pixmap_ul[0]); free(pixmap_ul[1]);
}

void Yajin::CreatePixmaps(void)
{
    GtkWidget* fixed;
    fixed = gtk_fixed_new();
    gtk_widget_set_usize(fixed, 100, 100);
    gtk_container_add(GTK_CONTAINER(window), fixed);
    gtk_widget_show(fixed);
    gtk_widget_realize(window);

    pixmap_u[0]  = CreatePixmap(XPM_U0);  pixmap_u[1]  = CreatePixmap(XPM_U1);
    pixmap_ur[0] = CreatePixmap(XPM_UR0); pixmap_ur[1] = CreatePixmap(XPM_UR1);
    pixmap_r[0]  = CreatePixmap(XPM_R0);  pixmap_r[1]  = CreatePixmap(XPM_R1);
    pixmap_dr[0] = CreatePixmap(XPM_DR0); pixmap_dr[1] = CreatePixmap(XPM_DR1);
    pixmap_d[0]  = CreatePixmap(XPM_D0);  pixmap_d[1]  = CreatePixmap(XPM_D1);
    pixmap_dl[0] = CreatePixmap(XPM_DL0); pixmap_dl[1] = CreatePixmap(XPM_DL1);
    pixmap_l[0]  = CreatePixmap(XPM_L0);  pixmap_l[1]  = CreatePixmap(XPM_L1);
    pixmap_ul[0] = CreatePixmap(XPM_UL0); pixmap_ul[1] = CreatePixmap(XPM_UL1);

    wpixmap = gtk_pixmap_new(pixmap_u[0]->pixmap, pixmap_u[0]->mask);
    gtk_fixed_put(GTK_FIXED(fixed), wpixmap, 0, 0);
    gtk_widget_show(wpixmap);
    return;
}

struct TXPM* Yajin::CreatePixmap(char** xpm_data)
{
    GdkGC*       gc;
    GtkStyle*    style;
    struct TXPM* xpm;
    style       = gtk_widget_get_default_style ();
    gc          = style->black_gc;	
    xpm         = (struct TXPM*)malloc(sizeof(struct TXPM));
    xpm->pixmap = gdk_pixmap_create_from_xpm_d(window->window, &xpm->mask,
                                               &style->bg[GTK_STATE_NORMAL],
                                               xpm_data);
    return xpm;
}

unsigned int Yajin::TimeoutAdd(int interval, GtkFunction func)
{
    return gtk_timeout_add(interval, func, window);
}

void Yajin::Move(int new_x, int new_y)
{
    flag_step ^= 1;
    if (x == new_x)
    {
        if (y < new_y)        PaintD();
        else if (y > new_y)   PaintU();
    }
    else if (x < new_x)
    {
        if (y == new_y)       PaintR();
        else if (y < new_y)   PaintDR();
        else /* y > new_y */  PaintUR();
    }
    else /* x > new_x */
    {
        if (y == new_y)       PaintL();
        else if (y < new_y)   PaintDL();
        else /* y > new_y */  PaintUL();
    }
    x = new_x;
    y = new_y;
    MoveWindow(x, y);
}

void Yajin::Paint(struct TXPM* xpm)
{
    gtk_widget_shape_combine_mask(window, xpm->mask, 0, 0);
    gtk_pixmap_set(GTK_PIXMAP(wpixmap), xpm->pixmap, xpm->mask);
}

void Yajin::PaintU(void)  { Paint(pixmap_u[flag_step]); }
void Yajin::PaintUR(void) { Paint(pixmap_ur[flag_step]); }
void Yajin::PaintR(void)  { Paint(pixmap_r[flag_step]); }
void Yajin::PaintDR(void) { Paint(pixmap_dr[flag_step]); }
void Yajin::PaintD(void)  { Paint(pixmap_d[flag_step]); }
void Yajin::PaintDL(void) { Paint(pixmap_dl[flag_step]); }
void Yajin::PaintL(void)  { Paint(pixmap_l[flag_step]); }
void Yajin::PaintUL(void) { Paint(pixmap_ul[flag_step]); }

void Yajin::MoveWindow(int x, int y)
{
    gtk_widget_set_uposition(window, x, y);
}
